from dataclasses import dataclass
from typing import List

import grpc

import depot_client.api.depot.core.v1.project_pb2 as project_pb2
import depot_client.api.depot.core.v1.project_pb2_grpc as project_pb2_grpc


@dataclass
class ProjectInfo:
    project_id: str
    organization_id: str
    name: str
    region_id: str
    # created_at: str
    # hardware: str


class ProjectService:
    def __init__(self, channel: grpc.Channel):
        self.stub = project_pb2_grpc.ProjectServiceStub(channel)

    def list_projects(self) -> List[ProjectInfo]:
        request = project_pb2.ListProjectsRequest()
        response = self.stub.ListProjects(request)
        return [
            ProjectInfo(
                project_id=project.project_id,
                organization_id=project.organization_id,
                name=project.name,
                region_id=project.region_id,
            )
            for project in response.projects
        ]

    def create_project(self, name: str, region_id: str) -> ProjectInfo:
        request = project_pb2.CreateProjectRequest(name=name, region_id=region_id)
        response = self.stub.CreateProject(request)
        return ProjectInfo(
            project_id=response.project.project_id,
            organization_id=response.project.organization_id,
            name=response.project.name,
            region_id=response.project.region_id,
        )

    def update_project(self, project_id: str, name: str, region_id: str) -> ProjectInfo:
        request = project_pb2.UpdateProjectRequest(
            project_id=project_id, name=name, region_id=region_id
        )
        response = self.stub.UpdateProject(request)
        return ProjectInfo(
            project_id=response.project.project_id,
            organization_id=response.project.organization_id,
            name=response.project.name,
            region_id=response.project.region_id,
        )

    def delete_project(self, project_id: str) -> None:
        request = project_pb2.DeleteProjectRequest(project_id=project_id)
        self.stub.DeleteProject(request)

    def reset_project(self, project_id: str) -> None:
        request = project_pb2.ResetProjectRequest(project_id=project_id)
        self.stub.ResetProject(request)

    def list_trust_policies():
        pass

    def add_trust_policy():
        pass

    def remove_trust_policy():
        pass

    def list_tokens():
        pass

    def create_token():
        pass

    def update_token():
        pass

    def delete_token():
        pass


class AsyncProjectService:
    def __init__(self, channel: grpc.Channel):
        self.stub = project_pb2_grpc.ProjectServiceStub(channel)

    async def list_projects(self) -> List[ProjectInfo]:
        request = project_pb2.ListProjectsRequest()
        response = await self.stub.ListProjects(request)
        return [
            ProjectInfo(
                project_id=project.project_id,
                organization_id=project.organization_id,
                name=project.name,
                region_id=project.region_id,
            )
            for project in response.projects
        ]

    async def create_project(self, name: str, region_id: str) -> ProjectInfo:
        request = project_pb2.CreateProjectRequest(name=name, region_id=region_id)
        response = await self.stub.CreateProject(request)
        return ProjectInfo(
            project_id=response.project.project_id,
            organization_id=response.project.organization_id,
            name=response.project.name,
            region_id=response.project.region_id,
        )

    async def update_project(
        self, project_id: str, name: str, region_id: str
    ) -> ProjectInfo:
        request = project_pb2.UpdateProjectRequest(
            project_id=project_id, name=name, region_id=region_id
        )
        response = await self.stub.UpdateProject(request)
        return ProjectInfo(
            project_id=response.project.project_id,
            organization_id=response.project.organization_id,
            name=response.project.name,
            region_id=response.project.region_id,
        )

    async def delete_project(self, project_id: str) -> None:
        request = project_pb2.DeleteProjectRequest(project_id=project_id)
        await self.stub.DeleteProject(request)

    async def reset_project(self, project_id: str) -> None:
        request = project_pb2.ResetProjectRequest(project_id=project_id)
        await self.stub.ResetProject(request)

    async def list_trust_policies():
        pass

    async def add_trust_policy():
        pass

    async def remove_trust_policy():
        pass

    async def list_tokens():
        pass

    async def create_token():
        pass

    async def update_token():
        pass

    async def delete_token():
        pass



import os
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

import grpc
import grpc.aio
from google.protobuf.timestamp_pb2 import Timestamp

from depot_client.build import AsyncBuildService, BuildService
from depot_client.buildkit import AsyncBuildKitService, BuildKitService, EndpointInfo
from depot_client.core_build import AsyncCoreBuildService, BuildInfo, CoreBuildService
from depot_client.project import AsyncProjectService, ProjectInfo, ProjectService

DEPOT_GRPC_HOST = "api.depot.dev"
DEPOT_GRPC_PORT = 443


@dataclass
class Endpoint(EndpointInfo):
    build_id: str
    platform: str
    buildkit: BuildKitService

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def close(self):
        self.buildkit.release_endpoint(self.build_id, self.platform)


@dataclass
class AsyncEndpoint(EndpointInfo):
    build_id: str
    platform: str
    buildkit: BuildKitService

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self.close()

    async def close(self):
        await self.buildkit.release_endpoint(self.build_id, self.platform)


class Build:
    def __init__(self, build_service, build_id: str, build_token: str):
        self.build_id = build_id
        self.build_token = build_token
        self.buildkit = BuildKitService(build_token)

    def close(self):
        self.buildkit.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def get_endpoint(self, platform: Optional[str] = None) -> Endpoint:
        endpoint = self.buildkit.get_endpoint(self.build_id, platform=platform)
        return Endpoint(
            endpoint=endpoint.endpoint,
            server_name=endpoint.server_name,
            cert=endpoint.cert,
            key=endpoint.key,
            ca_cert=endpoint.ca_cert,
            build_id=self.build_id,
            platform=platform,
            buildkit=self.buildkit,
        )


class AsyncBuild:
    def __init__(self, build_service, build_id: str, build_token: str):
        self.build_id = build_id
        self.build_token = build_token
        self.buildkit = AsyncBuildKitService(build_token)

    async def close(self):
        await self.buildkit.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self.close()

    async def get_endpoint(self, platform: Optional[str] = None) -> AsyncEndpoint:
        endpoint = await self.buildkit.get_endpoint(self.build_id, platform=platform)
        return AsyncEndpoint(
            endpoint=endpoint.endpoint,
            server_name=endpoint.server_name,
            cert=endpoint.cert,
            key=endpoint.key,
            ca_cert=endpoint.ca_cert,
            build_id=self.build_id,
            platform=platform,
            buildkit=self.buildkit,
        )


class BaseClient:
    def _create_channel_credentials(self) -> grpc.ChannelCredentials:
        channel_creds = grpc.ssl_channel_credentials()
        token = os.getenv("DEPOT_API_TOKEN")
        call_creds = grpc.access_token_call_credentials(token)
        return grpc.composite_channel_credentials(channel_creds, call_creds)

    def _proto_to_datetime(self, timestamp: Timestamp) -> datetime:
        return datetime.fromtimestamp(timestamp.seconds + timestamp.nanos / 1e9)


class Client(BaseClient):
    def __init__(
        self,
        host: str = DEPOT_GRPC_HOST,
        port: int = DEPOT_GRPC_PORT,
    ):
        credentials = self._create_channel_credentials()
        self.channel = grpc.secure_channel(f"{host}:{port}", credentials)
        self.build = BuildService(self.channel)
        self.core_build = CoreBuildService(self.channel)
        self.project = ProjectService(self.channel)

    def close(self):
        self.channel.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def list_projects(self) -> List[ProjectInfo]:
        return self.project.list_projects()

    def create_build(self, project_id: str) -> Build:
        build_id, build_token = self.build.create_build(project_id)
        return Build(self.build, build_id=build_id, build_token=build_token)

    def finish_build(self, build_id: str, error: Optional[str] = None) -> None:
        return self.build.finish_build(build_id, error=error)

    def share_build(self, build_id: str) -> str:
        return self.core_build.share_build(build_id)

    def stop_sharing_build(self, build_id: str) -> None:
        return self.core_build.stop_sharing_build(build_id)

    def get_build(self, build_id: str) -> BuildInfo:
        return self.core_build.get_build(build_id)

    def list_builds(
        self,
        project_id: str,
    ) -> List[BuildInfo]:
        return self.core_build.list_builds(project_id)

    def create_endpoint(
        self, build_id: str, platform: Optional[str] = None
    ) -> Endpoint:
        return self.buildkit.create_endpoint(build_id, platform)


class AsyncClient(BaseClient):
    def __init__(
        self,
        host: str = DEPOT_GRPC_HOST,
        port: int = DEPOT_GRPC_PORT,
    ):
        credentials = self._create_channel_credentials()
        self.channel = grpc.aio.secure_channel(f"{host}:{port}", credentials)
        self.build = AsyncBuildService(self.channel)
        self.core_build = AsyncCoreBuildService(self.channel)
        self.project = AsyncProjectService(self.channel)

    async def close(self):
        await self.channel.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self.close()

    async def list_projects(self) -> List[ProjectInfo]:
        return await self.project.list_projects()

    async def create_build(self, project_id: str) -> AsyncBuild:
        build_id, build_token = await self.build.create_build(project_id)
        return AsyncBuild(self.build, build_id=build_id, build_token=build_token)

    async def finish_build(self, build_id: str, error: Optional[str] = None) -> None:
        return await self.build.finish_build(build_id, error=error)

    async def share_build(self, build_id: str) -> str:
        return await self.core_build.share_build(build_id)

    async def stop_sharing_build(self, build_id: str) -> None:
        return await self.core_build.stop_sharing_build(build_id)

    async def get_build(self, build_id: str) -> BuildInfo:
        return await self.core_build.get_build(build_id)

    async def list_builds(
        self,
        project_id: str,
    ) -> List[BuildInfo]:
        return await self.core_build.list_builds(project_id)


def _main():
    with Client() as client:
        client.list_projects()
        project_id = "749dxclhrj"
        client.list_builds(project_id)
        with client.create_build(project_id) as build:
            with build.get_endpoint() as endpoint:
                print(repr(endpoint))
                assert isinstance(endpoint.cert, str)
                assert isinstance(endpoint.key, str)
                assert isinstance(endpoint.ca_cert, str)


async def _async_main():
    async with AsyncClient() as client:
        await client.list_projects()
        project_id = "749dxclhrj"
        await client.list_builds(project_id)
        async with await client.create_build(project_id) as build:
            async with await build.get_endpoint() as endpoint:
                print(repr(endpoint))
                assert isinstance(endpoint.cert, str)
                assert isinstance(endpoint.key, str)
                assert isinstance(endpoint.ca_cert, str)


if __name__ == "__main__":
    _main()
    import asyncio

    asyncio.run(_async_main())



from typing import Optional, Tuple

import grpc

from depot_client.api.depot.build.v1.build_pb2 import (
    CreateBuildRequest,
    FinishBuildRequest,
)
from depot_client.api.depot.build.v1.build_pb2_grpc import BuildServiceStub


class BuildService:
    def __init__(self, channel: grpc.Channel):
        self.stub = BuildServiceStub(channel)

    def create_build(self, project_id: str) -> Tuple[str, str]:
        request = CreateBuildRequest(project_id=project_id)
        response = self.stub.CreateBuild(request)
        return response.build_id, response.build_token

    def finish_build(self, build_id: str, error: Optional[str] = None) -> None:
        if error is None:
            request = FinishBuildRequest(
                build_id=build_id,
                success=FinishBuildRequest.BuildSuccess(),
            )
        else:
            request = FinishBuildRequest(
                build_id=build_id,
                error=FinishBuildRequest.BuildError(error),
            )

        self.stub.FinishBuild(request)


class AsyncBuildService:
    def __init__(self, channel: grpc.Channel):
        self.stub = BuildServiceStub(channel)

    async def create_build(self, project_id: str) -> Tuple[str, str]:
        request = CreateBuildRequest(project_id=project_id)
        response = await self.stub.CreateBuild(request)
        return response.build_id, response.build_token

    async def finish_build(self, build_id: str, error: Optional[str] = None) -> None:
        if error is None:
            request = FinishBuildRequest(
                build_id=build_id,
                success=FinishBuildRequest.BuildSuccess(),
            )
        else:
            request = FinishBuildRequest(
                build_id=build_id,
                error=FinishBuildRequest.BuildError(error),
            )

        await self.stub.FinishBuild(request)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from depot_client.api.depot.core.v1 import build_pb2 as depot_dot_core_dot_v1_dot_build__pb2

GRPC_GENERATED_VERSION = '1.64.0'
GRPC_VERSION = grpc.__version__
EXPECTED_ERROR_RELEASE = '1.65.0'
SCHEDULED_RELEASE_DATE = 'June 25, 2024'
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    warnings.warn(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in depot/core/v1/build_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
        + f' This warning will become an error in {EXPECTED_ERROR_RELEASE},'
        + f' scheduled for release on {SCHEDULED_RELEASE_DATE}.',
        RuntimeWarning
    )


class BuildServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ShareBuild = channel.unary_unary(
                '/depot.core.v1.BuildService/ShareBuild',
                request_serializer=depot_dot_core_dot_v1_dot_build__pb2.ShareBuildRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_build__pb2.ShareBuildResponse.FromString,
                _registered_method=True)
        self.StopSharingBuild = channel.unary_unary(
                '/depot.core.v1.BuildService/StopSharingBuild',
                request_serializer=depot_dot_core_dot_v1_dot_build__pb2.StopSharingBuildRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_build__pb2.StopSharingBuildResponse.FromString,
                _registered_method=True)
        self.ListBuilds = channel.unary_unary(
                '/depot.core.v1.BuildService/ListBuilds',
                request_serializer=depot_dot_core_dot_v1_dot_build__pb2.ListBuildsRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_build__pb2.ListBuildsResponse.FromString,
                _registered_method=True)
        self.GetBuild = channel.unary_unary(
                '/depot.core.v1.BuildService/GetBuild',
                request_serializer=depot_dot_core_dot_v1_dot_build__pb2.GetBuildRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_build__pb2.GetBuildResponse.FromString,
                _registered_method=True)


class BuildServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ShareBuild(self, request, context):
        """Share a URL to a build with users outside of your organization
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopSharingBuild(self, request, context):
        """Stop sharing a build
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListBuilds(self, request, context):
        """List the builds in your organization
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBuild(self, request, context):
        """Get the build metadata of a build
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_BuildServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ShareBuild': grpc.unary_unary_rpc_method_handler(
                    servicer.ShareBuild,
                    request_deserializer=depot_dot_core_dot_v1_dot_build__pb2.ShareBuildRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_build__pb2.ShareBuildResponse.SerializeToString,
            ),
            'StopSharingBuild': grpc.unary_unary_rpc_method_handler(
                    servicer.StopSharingBuild,
                    request_deserializer=depot_dot_core_dot_v1_dot_build__pb2.StopSharingBuildRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_build__pb2.StopSharingBuildResponse.SerializeToString,
            ),
            'ListBuilds': grpc.unary_unary_rpc_method_handler(
                    servicer.ListBuilds,
                    request_deserializer=depot_dot_core_dot_v1_dot_build__pb2.ListBuildsRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_build__pb2.ListBuildsResponse.SerializeToString,
            ),
            'GetBuild': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBuild,
                    request_deserializer=depot_dot_core_dot_v1_dot_build__pb2.GetBuildRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_build__pb2.GetBuildResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'depot.core.v1.BuildService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('depot.core.v1.BuildService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class BuildService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ShareBuild(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.BuildService/ShareBuild',
            depot_dot_core_dot_v1_dot_build__pb2.ShareBuildRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_build__pb2.ShareBuildResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StopSharingBuild(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.BuildService/StopSharingBuild',
            depot_dot_core_dot_v1_dot_build__pb2.StopSharingBuildRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_build__pb2.StopSharingBuildResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListBuilds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.BuildService/ListBuilds',
            depot_dot_core_dot_v1_dot_build__pb2.ListBuildsRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_build__pb2.ListBuildsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetBuild(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.BuildService/GetBuild',
            depot_dot_core_dot_v1_dot_build__pb2.GetBuildRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_build__pb2.GetBuildResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: depot/buildkit/v1/buildkit.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n depot/buildkit/v1/buildkit.proto\x12\x11\x64\x65pot.buildkit.v1\"U\n\x12GetEndpointRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12-\n\x08platform\x18\x02 \x01(\x0e\x32\x1b.depot.buildkit.v1.Platform\"\xf5\x02\n\x13GetEndpointResponse\x12K\n\x07pending\x18\x01 \x01(\x0b\x32\x38.depot.buildkit.v1.GetEndpointResponse.PendingConnectionH\x00\x12I\n\x06\x61\x63tive\x18\x02 \x01(\x0b\x32\x37.depot.buildkit.v1.GetEndpointResponse.ActiveConnectionH\x00\x1a\x13\n\x11PendingConnection\x1a\xa2\x01\n\x10\x41\x63tiveConnection\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12\x13\n\x0bserver_name\x18\x02 \x01(\t\x12\x30\n\x04\x63\x65rt\x18\x03 \x01(\x0b\x32\".depot.buildkit.v1.CertificatePair\x12\x35\n\x07\x63\x61_cert\x18\x04 \x01(\x0b\x32$.depot.buildkit.v1.PublicCertificateB\x0c\n\nconnection\"Y\n\x16ReleaseEndpointRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12-\n\x08platform\x18\x02 \x01(\x0e\x32\x1b.depot.buildkit.v1.Platform\"\x19\n\x17ReleaseEndpointResponse\"!\n\x11PublicCertificate\x12\x0c\n\x04\x63\x65rt\x18\x01 \x01(\t\"\x19\n\nPrivateKey\x12\x0b\n\x03key\x18\x01 \x01(\t\"q\n\x0f\x43\x65rtificatePair\x12\x32\n\x04\x63\x65rt\x18\x01 \x01(\x0b\x32$.depot.buildkit.v1.PublicCertificate\x12*\n\x03key\x18\x02 \x01(\x0b\x32\x1d.depot.buildkit.v1.PrivateKey\"V\n\x13ReportHealthRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12-\n\x08platform\x18\x02 \x01(\x0e\x32\x1b.depot.buildkit.v1.Platform\"\x16\n\x14ReportHealthResponse*L\n\x08Platform\x12\x18\n\x14PLATFORM_UNSPECIFIED\x10\x00\x12\x12\n\x0ePLATFORM_AMD64\x10\x01\x12\x12\n\x0ePLATFORM_ARM64\x10\x02\x32\xbe\x02\n\x0f\x42uildKitService\x12^\n\x0bGetEndpoint\x12%.depot.buildkit.v1.GetEndpointRequest\x1a&.depot.buildkit.v1.GetEndpointResponse0\x01\x12\x61\n\x0cReportHealth\x12&.depot.buildkit.v1.ReportHealthRequest\x1a\'.depot.buildkit.v1.ReportHealthResponse(\x01\x12h\n\x0fReleaseEndpoint\x12).depot.buildkit.v1.ReleaseEndpointRequest\x1a*.depot.buildkit.v1.ReleaseEndpointResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'depot.buildkit.v1.buildkit_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_PLATFORM']._serialized_start=925
  _globals['_PLATFORM']._serialized_end=1001
  _globals['_GETENDPOINTREQUEST']._serialized_start=55
  _globals['_GETENDPOINTREQUEST']._serialized_end=140
  _globals['_GETENDPOINTRESPONSE']._serialized_start=143
  _globals['_GETENDPOINTRESPONSE']._serialized_end=516
  _globals['_GETENDPOINTRESPONSE_PENDINGCONNECTION']._serialized_start=318
  _globals['_GETENDPOINTRESPONSE_PENDINGCONNECTION']._serialized_end=337
  _globals['_GETENDPOINTRESPONSE_ACTIVECONNECTION']._serialized_start=340
  _globals['_GETENDPOINTRESPONSE_ACTIVECONNECTION']._serialized_end=502
  _globals['_RELEASEENDPOINTREQUEST']._serialized_start=518
  _globals['_RELEASEENDPOINTREQUEST']._serialized_end=607
  _globals['_RELEASEENDPOINTRESPONSE']._serialized_start=609
  _globals['_RELEASEENDPOINTRESPONSE']._serialized_end=634
  _globals['_PUBLICCERTIFICATE']._serialized_start=636
  _globals['_PUBLICCERTIFICATE']._serialized_end=669
  _globals['_PRIVATEKEY']._serialized_start=671
  _globals['_PRIVATEKEY']._serialized_end=696
  _globals['_CERTIFICATEPAIR']._serialized_start=698
  _globals['_CERTIFICATEPAIR']._serialized_end=811
  _globals['_REPORTHEALTHREQUEST']._serialized_start=813
  _globals['_REPORTHEALTHREQUEST']._serialized_end=899
  _globals['_REPORTHEALTHRESPONSE']._serialized_start=901
  _globals['_REPORTHEALTHRESPONSE']._serialized_end=923
  _globals['_BUILDKITSERVICE']._serialized_start=1004
  _globals['_BUILDKITSERVICE']._serialized_end=1322
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: depot/core/v1/project.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x64\x65pot/core/v1/project.proto\x12\rdepot.core.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\xe4\x01\n\x07Project\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tregion_id\x18\x04 \x01(\t\x12.\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0c\x63\x61\x63he_policy\x18\x06 \x01(\x0b\x32\x1a.depot.core.v1.CachePolicy\x12)\n\x08hardware\x18\x07 \x01(\x0e\x32\x17.depot.core.v1.Hardware\"\x15\n\x13ListProjectsRequest\"@\n\x14ListProjectsResponse\x12(\n\x08projects\x18\x01 \x03(\x0b\x32\x16.depot.core.v1.Project\"\'\n\x11GetProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"=\n\x12GetProjectResponse\x12\'\n\x07project\x18\x01 \x01(\x0b\x32\x16.depot.core.v1.Project\"\xee\x01\n\x14\x43reateProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1c\n\x0forganization_id\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x11\n\tregion_id\x18\x03 \x01(\t\x12\x35\n\x0c\x63\x61\x63he_policy\x18\x04 \x01(\x0b\x32\x1a.depot.core.v1.CachePolicyH\x01\x88\x01\x01\x12.\n\x08hardware\x18\x05 \x01(\x0e\x32\x17.depot.core.v1.HardwareH\x02\x88\x01\x01\x42\x12\n\x10_organization_idB\x0f\n\r_cache_policyB\x0b\n\t_hardware\"@\n\x15\x43reateProjectResponse\x12\'\n\x07project\x18\x01 \x01(\x0b\x32\x16.depot.core.v1.Project\"\xf1\x01\n\x14UpdateProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x11\n\x04name\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x16\n\tregion_id\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x35\n\x0c\x63\x61\x63he_policy\x18\x04 \x01(\x0b\x32\x1a.depot.core.v1.CachePolicyH\x02\x88\x01\x01\x12.\n\x08hardware\x18\x05 \x01(\x0e\x32\x17.depot.core.v1.HardwareH\x03\x88\x01\x01\x42\x07\n\x05_nameB\x0c\n\n_region_idB\x0f\n\r_cache_policyB\x0b\n\t_hardware\"@\n\x15UpdateProjectResponse\x12\'\n\x07project\x18\x01 \x01(\x0b\x32\x16.depot.core.v1.Project\"*\n\x14\x44\x65leteProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"\x17\n\x15\x44\x65leteProjectResponse\")\n\x13ResetProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"\x16\n\x14ResetProjectResponse\"]\n\x0b\x43\x61\x63hePolicy\x12\x1b\n\nkeep_bytes\x18\x01 \x01(\x03\x42\x02\x18\x01H\x00\x88\x01\x01\x12\x11\n\tkeep_days\x18\x02 \x01(\x05\x12\x0f\n\x07keep_gb\x18\x03 \x01(\x05\x42\r\n\x0b_keep_bytes\".\n\x18ListTrustPoliciesRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"O\n\x19ListTrustPoliciesResponse\x12\x32\n\x0etrust_policies\x18\x01 \x03(\x0b\x32\x1a.depot.core.v1.TrustPolicy\"\xe0\x01\n\x15\x41\x64\x64TrustPolicyRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x39\n\tbuildkite\x18\x02 \x01(\x0b\x32$.depot.core.v1.TrustPolicy.BuildkiteH\x00\x12\x37\n\x08\x63ircleci\x18\x03 \x01(\x0b\x32#.depot.core.v1.TrustPolicy.CircleCIH\x00\x12\x33\n\x06github\x18\x04 \x01(\x0b\x32!.depot.core.v1.TrustPolicy.GitHubH\x00\x42\n\n\x08provider\"J\n\x16\x41\x64\x64TrustPolicyResponse\x12\x30\n\x0ctrust_policy\x18\x01 \x01(\x0b\x32\x1a.depot.core.v1.TrustPolicy\"G\n\x18RemoveTrustPolicyRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x17\n\x0ftrust_policy_id\x18\x02 \x01(\t\"\x1b\n\x19RemoveTrustPolicyResponse\"\x8f\x03\n\x0bTrustPolicy\x12\x17\n\x0ftrust_policy_id\x18\x01 \x01(\t\x12\x39\n\tbuildkite\x18\x02 \x01(\x0b\x32$.depot.core.v1.TrustPolicy.BuildkiteH\x00\x12\x37\n\x08\x63ircleci\x18\x03 \x01(\x0b\x32#.depot.core.v1.TrustPolicy.CircleCIH\x00\x12\x33\n\x06github\x18\x04 \x01(\x0b\x32!.depot.core.v1.TrustPolicy.GitHubH\x00\x1a\x36\n\x06GitHub\x12\x18\n\x10repository_owner\x18\x01 \x01(\t\x12\x12\n\nrepository\x18\x02 \x01(\t\x1a;\n\x08\x43ircleCI\x12\x19\n\x11organization_uuid\x18\x01 \x01(\t\x12\x14\n\x0cproject_uuid\x18\x02 \x01(\t\x1a=\n\tBuildkite\x12\x19\n\x11organization_slug\x18\x01 \x01(\t\x12\x15\n\rpipeline_slug\x18\x02 \x01(\tB\n\n\x08provider\"\'\n\x11ListTokensRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"}\n\x12ListTokensResponse\x12\x37\n\x06tokens\x18\x01 \x03(\x0b\x32\'.depot.core.v1.ListTokensResponse.Token\x1a.\n\x05Token\x12\x10\n\x08token_id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\"=\n\x12\x43reateTokenRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\"7\n\x13\x43reateTokenResponse\x12\x10\n\x08token_id\x18\x01 \x01(\t\x12\x0e\n\x06secret\x18\x02 \x01(\t\";\n\x12UpdateTokenRequest\x12\x10\n\x08token_id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\"\x15\n\x13UpdateTokenResponse\"&\n\x12\x44\x65leteTokenRequest\x12\x10\n\x08token_id\x18\x01 \x01(\t\"\x15\n\x13\x44\x65leteTokenResponse*\x98\x01\n\x08Hardware\x12\x18\n\x14HARDWARE_UNSPECIFIED\x10\x00\x12\x12\n\x0eHARDWARE_16X32\x10\x01\x12\x10\n\x0cHARDWARE_4X4\x10\x02\x12\x10\n\x0cHARDWARE_8X8\x10\x03\x12\x11\n\rHARDWARE_8X16\x10\x04\x12\x12\n\x0eHARDWARE_32X64\x10\x05\x12\x13\n\x0fHARDWARE_64X128\x10\x06\x32\xc7\t\n\x0eProjectService\x12Y\n\x0cListProjects\x12\".depot.core.v1.ListProjectsRequest\x1a#.depot.core.v1.ListProjectsResponse\"\x00\x12S\n\nGetProject\x12 .depot.core.v1.GetProjectRequest\x1a!.depot.core.v1.GetProjectResponse\"\x00\x12\\\n\rCreateProject\x12#.depot.core.v1.CreateProjectRequest\x1a$.depot.core.v1.CreateProjectResponse\"\x00\x12\\\n\rUpdateProject\x12#.depot.core.v1.UpdateProjectRequest\x1a$.depot.core.v1.UpdateProjectResponse\"\x00\x12\\\n\rDeleteProject\x12#.depot.core.v1.DeleteProjectRequest\x1a$.depot.core.v1.DeleteProjectResponse\"\x00\x12Y\n\x0cResetProject\x12\".depot.core.v1.ResetProjectRequest\x1a#.depot.core.v1.ResetProjectResponse\"\x00\x12h\n\x11ListTrustPolicies\x12\'.depot.core.v1.ListTrustPoliciesRequest\x1a(.depot.core.v1.ListTrustPoliciesResponse\"\x00\x12_\n\x0e\x41\x64\x64TrustPolicy\x12$.depot.core.v1.AddTrustPolicyRequest\x1a%.depot.core.v1.AddTrustPolicyResponse\"\x00\x12h\n\x11RemoveTrustPolicy\x12\'.depot.core.v1.RemoveTrustPolicyRequest\x1a(.depot.core.v1.RemoveTrustPolicyResponse\"\x00\x12S\n\nListTokens\x12 .depot.core.v1.ListTokensRequest\x1a!.depot.core.v1.ListTokensResponse\"\x00\x12V\n\x0b\x43reateToken\x12!.depot.core.v1.CreateTokenRequest\x1a\".depot.core.v1.CreateTokenResponse\"\x00\x12V\n\x0bUpdateToken\x12!.depot.core.v1.UpdateTokenRequest\x1a\".depot.core.v1.UpdateTokenResponse\"\x00\x12V\n\x0b\x44\x65leteToken\x12!.depot.core.v1.DeleteTokenRequest\x1a\".depot.core.v1.DeleteTokenResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'depot.core.v1.project_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CACHEPOLICY'].fields_by_name['keep_bytes']._loaded_options = None
  _globals['_CACHEPOLICY'].fields_by_name['keep_bytes']._serialized_options = b'\030\001'
  _globals['_HARDWARE']._serialized_start=2723
  _globals['_HARDWARE']._serialized_end=2875
  _globals['_PROJECT']._serialized_start=80
  _globals['_PROJECT']._serialized_end=308
  _globals['_LISTPROJECTSREQUEST']._serialized_start=310
  _globals['_LISTPROJECTSREQUEST']._serialized_end=331
  _globals['_LISTPROJECTSRESPONSE']._serialized_start=333
  _globals['_LISTPROJECTSRESPONSE']._serialized_end=397
  _globals['_GETPROJECTREQUEST']._serialized_start=399
  _globals['_GETPROJECTREQUEST']._serialized_end=438
  _globals['_GETPROJECTRESPONSE']._serialized_start=440
  _globals['_GETPROJECTRESPONSE']._serialized_end=501
  _globals['_CREATEPROJECTREQUEST']._serialized_start=504
  _globals['_CREATEPROJECTREQUEST']._serialized_end=742
  _globals['_CREATEPROJECTRESPONSE']._serialized_start=744
  _globals['_CREATEPROJECTRESPONSE']._serialized_end=808
  _globals['_UPDATEPROJECTREQUEST']._serialized_start=811
  _globals['_UPDATEPROJECTREQUEST']._serialized_end=1052
  _globals['_UPDATEPROJECTRESPONSE']._serialized_start=1054
  _globals['_UPDATEPROJECTRESPONSE']._serialized_end=1118
  _globals['_DELETEPROJECTREQUEST']._serialized_start=1120
  _globals['_DELETEPROJECTREQUEST']._serialized_end=1162
  _globals['_DELETEPROJECTRESPONSE']._serialized_start=1164
  _globals['_DELETEPROJECTRESPONSE']._serialized_end=1187
  _globals['_RESETPROJECTREQUEST']._serialized_start=1189
  _globals['_RESETPROJECTREQUEST']._serialized_end=1230
  _globals['_RESETPROJECTRESPONSE']._serialized_start=1232
  _globals['_RESETPROJECTRESPONSE']._serialized_end=1254
  _globals['_CACHEPOLICY']._serialized_start=1256
  _globals['_CACHEPOLICY']._serialized_end=1349
  _globals['_LISTTRUSTPOLICIESREQUEST']._serialized_start=1351
  _globals['_LISTTRUSTPOLICIESREQUEST']._serialized_end=1397
  _globals['_LISTTRUSTPOLICIESRESPONSE']._serialized_start=1399
  _globals['_LISTTRUSTPOLICIESRESPONSE']._serialized_end=1478
  _globals['_ADDTRUSTPOLICYREQUEST']._serialized_start=1481
  _globals['_ADDTRUSTPOLICYREQUEST']._serialized_end=1705
  _globals['_ADDTRUSTPOLICYRESPONSE']._serialized_start=1707
  _globals['_ADDTRUSTPOLICYRESPONSE']._serialized_end=1781
  _globals['_REMOVETRUSTPOLICYREQUEST']._serialized_start=1783
  _globals['_REMOVETRUSTPOLICYREQUEST']._serialized_end=1854
  _globals['_REMOVETRUSTPOLICYRESPONSE']._serialized_start=1856
  _globals['_REMOVETRUSTPOLICYRESPONSE']._serialized_end=1883
  _globals['_TRUSTPOLICY']._serialized_start=1886
  _globals['_TRUSTPOLICY']._serialized_end=2285
  _globals['_TRUSTPOLICY_GITHUB']._serialized_start=2095
  _globals['_TRUSTPOLICY_GITHUB']._serialized_end=2149
  _globals['_TRUSTPOLICY_CIRCLECI']._serialized_start=2151
  _globals['_TRUSTPOLICY_CIRCLECI']._serialized_end=2210
  _globals['_TRUSTPOLICY_BUILDKITE']._serialized_start=2212
  _globals['_TRUSTPOLICY_BUILDKITE']._serialized_end=2273
  _globals['_LISTTOKENSREQUEST']._serialized_start=2287
  _globals['_LISTTOKENSREQUEST']._serialized_end=2326
  _globals['_LISTTOKENSRESPONSE']._serialized_start=2328
  _globals['_LISTTOKENSRESPONSE']._serialized_end=2453
  _globals['_LISTTOKENSRESPONSE_TOKEN']._serialized_start=2407
  _globals['_LISTTOKENSRESPONSE_TOKEN']._serialized_end=2453
  _globals['_CREATETOKENREQUEST']._serialized_start=2455
  _globals['_CREATETOKENREQUEST']._serialized_end=2516
  _globals['_CREATETOKENRESPONSE']._serialized_start=2518
  _globals['_CREATETOKENRESPONSE']._serialized_end=2573
  _globals['_UPDATETOKENREQUEST']._serialized_start=2575
  _globals['_UPDATETOKENREQUEST']._serialized_end=2634
  _globals['_UPDATETOKENRESPONSE']._serialized_start=2636
  _globals['_UPDATETOKENRESPONSE']._serialized_end=2657
  _globals['_DELETETOKENREQUEST']._serialized_start=2659
  _globals['_DELETETOKENREQUEST']._serialized_end=2697
  _globals['_DELETETOKENRESPONSE']._serialized_start=2699
  _globals['_DELETETOKENRESPONSE']._serialized_end=2720
  _globals['_PROJECTSERVICE']._serialized_start=2878
  _globals['_PROJECTSERVICE']._serialized_end=4101
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass

from depot_client.api.depot.core.v1.build_pb2 import (
    GetBuildRequest,
    ListBuildsRequest,
    ShareBuildRequest,
    StopSharingBuildRequest,
)
from depot_client.api.depot.core.v1.build_pb2_grpc import BuildServiceStub


@dataclass
class BuildInfo:
    build_id: str
    status: str


class CoreBuildService:
    def __init__(self, channel):
        self.stub = BuildServiceStub(channel)

    def share_build(self, build_id: str) -> str:
        request = ShareBuildRequest(build_id)
        return self.stub.ShareBuild(request).share_url

    def stop_sharing_build(self, build_id: str) -> None:
        request = StopSharingBuildRequest(build_id)
        self.stub.StopSharingBuild(request)

    def list_builds(self, project_id: str) -> list[BuildInfo]:
        # TODO: implement pagination with page_size/page_token
        request = ListBuildsRequest(project_id=project_id)
        response = self.stub.ListBuilds(request)
        return [BuildInfo(build.build_id, build.status) for build in response.builds]

    def get_build(self, build_id: str) -> BuildInfo:
        request = GetBuildRequest(build_id=build_id)
        response = self.stub.GetBuild(request)
        return BuildInfo(response.build.build_id, response.build.status)


class AsyncCoreBuildService:
    def __init__(self, channel):
        self.stub = BuildServiceStub(channel)

    async def share_build(self, build_id: str) -> str:
        request = ShareBuildRequest(build_id)
        response = await self.stub.ShareBuild(request)
        return response.share_url

    async def stop_sharing_build(self, build_id: str) -> None:
        request = StopSharingBuildRequest(build_id)
        await self.stub.StopSharingBuild(request)

    async def list_builds(self, project_id: str) -> list[BuildInfo]:
        # TODO: implement pagination with page_size/page_token
        request = ListBuildsRequest(project_id=project_id)
        response = await self.stub.ListBuilds(request)
        return [BuildInfo(build.build_id, build.status) for build in response.builds]

    async def get_build(self, build_id: str) -> BuildInfo:
        request = GetBuildRequest(build_id)
        response = await self.stub.GetBuild(request)
        return BuildInfo(response.build.build_id, response.build.status)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from depot_client.api.depot.core.v1 import project_pb2 as depot_dot_core_dot_v1_dot_project__pb2

GRPC_GENERATED_VERSION = '1.64.0'
GRPC_VERSION = grpc.__version__
EXPECTED_ERROR_RELEASE = '1.65.0'
SCHEDULED_RELEASE_DATE = 'June 25, 2024'
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    warnings.warn(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in depot/core/v1/project_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
        + f' This warning will become an error in {EXPECTED_ERROR_RELEASE},'
        + f' scheduled for release on {SCHEDULED_RELEASE_DATE}.',
        RuntimeWarning
    )


class ProjectServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListProjects = channel.unary_unary(
                '/depot.core.v1.ProjectService/ListProjects',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.ListProjectsRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ListProjectsResponse.FromString,
                _registered_method=True)
        self.GetProject = channel.unary_unary(
                '/depot.core.v1.ProjectService/GetProject',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.GetProjectRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.GetProjectResponse.FromString,
                _registered_method=True)
        self.CreateProject = channel.unary_unary(
                '/depot.core.v1.ProjectService/CreateProject',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.CreateProjectRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.CreateProjectResponse.FromString,
                _registered_method=True)
        self.UpdateProject = channel.unary_unary(
                '/depot.core.v1.ProjectService/UpdateProject',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateProjectRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateProjectResponse.FromString,
                _registered_method=True)
        self.DeleteProject = channel.unary_unary(
                '/depot.core.v1.ProjectService/DeleteProject',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteProjectRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteProjectResponse.FromString,
                _registered_method=True)
        self.ResetProject = channel.unary_unary(
                '/depot.core.v1.ProjectService/ResetProject',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.ResetProjectRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ResetProjectResponse.FromString,
                _registered_method=True)
        self.ListTrustPolicies = channel.unary_unary(
                '/depot.core.v1.ProjectService/ListTrustPolicies',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.ListTrustPoliciesRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ListTrustPoliciesResponse.FromString,
                _registered_method=True)
        self.AddTrustPolicy = channel.unary_unary(
                '/depot.core.v1.ProjectService/AddTrustPolicy',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.AddTrustPolicyRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.AddTrustPolicyResponse.FromString,
                _registered_method=True)
        self.RemoveTrustPolicy = channel.unary_unary(
                '/depot.core.v1.ProjectService/RemoveTrustPolicy',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.RemoveTrustPolicyRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.RemoveTrustPolicyResponse.FromString,
                _registered_method=True)
        self.ListTokens = channel.unary_unary(
                '/depot.core.v1.ProjectService/ListTokens',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.ListTokensRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ListTokensResponse.FromString,
                _registered_method=True)
        self.CreateToken = channel.unary_unary(
                '/depot.core.v1.ProjectService/CreateToken',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.CreateTokenRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.CreateTokenResponse.FromString,
                _registered_method=True)
        self.UpdateToken = channel.unary_unary(
                '/depot.core.v1.ProjectService/UpdateToken',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateTokenRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateTokenResponse.FromString,
                _registered_method=True)
        self.DeleteToken = channel.unary_unary(
                '/depot.core.v1.ProjectService/DeleteToken',
                request_serializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteTokenRequest.SerializeToString,
                response_deserializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteTokenResponse.FromString,
                _registered_method=True)


class ProjectServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListProjects(self, request, context):
        """List all projects
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProject(self, request, context):
        """Get a project
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateProject(self, request, context):
        """Create a new project
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateProject(self, request, context):
        """Update a project
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteProject(self, request, context):
        """Delete a project
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetProject(self, request, context):
        """Reset a project will terminate all machines and delete all cached data.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListTrustPolicies(self, request, context):
        """List project's trust policies.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddTrustPolicy(self, request, context):
        """Add a trust policy to a project.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveTrustPolicy(self, request, context):
        """Remove a trust policy from a project.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListTokens(self, request, context):
        """List project's API tokens.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateToken(self, request, context):
        """Create a project API token.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateToken(self, request, context):
        """Create a project API token.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteToken(self, request, context):
        """Delete project API token.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ProjectServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListProjects': grpc.unary_unary_rpc_method_handler(
                    servicer.ListProjects,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ListProjectsRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.ListProjectsResponse.SerializeToString,
            ),
            'GetProject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProject,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.GetProjectRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.GetProjectResponse.SerializeToString,
            ),
            'CreateProject': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateProject,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.CreateProjectRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.CreateProjectResponse.SerializeToString,
            ),
            'UpdateProject': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateProject,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateProjectRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateProjectResponse.SerializeToString,
            ),
            'DeleteProject': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteProject,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteProjectRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteProjectResponse.SerializeToString,
            ),
            'ResetProject': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetProject,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ResetProjectRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.ResetProjectResponse.SerializeToString,
            ),
            'ListTrustPolicies': grpc.unary_unary_rpc_method_handler(
                    servicer.ListTrustPolicies,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ListTrustPoliciesRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.ListTrustPoliciesResponse.SerializeToString,
            ),
            'AddTrustPolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.AddTrustPolicy,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.AddTrustPolicyRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.AddTrustPolicyResponse.SerializeToString,
            ),
            'RemoveTrustPolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveTrustPolicy,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.RemoveTrustPolicyRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.RemoveTrustPolicyResponse.SerializeToString,
            ),
            'ListTokens': grpc.unary_unary_rpc_method_handler(
                    servicer.ListTokens,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.ListTokensRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.ListTokensResponse.SerializeToString,
            ),
            'CreateToken': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateToken,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.CreateTokenRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.CreateTokenResponse.SerializeToString,
            ),
            'UpdateToken': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateToken,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateTokenRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.UpdateTokenResponse.SerializeToString,
            ),
            'DeleteToken': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteToken,
                    request_deserializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteTokenRequest.FromString,
                    response_serializer=depot_dot_core_dot_v1_dot_project__pb2.DeleteTokenResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'depot.core.v1.ProjectService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('depot.core.v1.ProjectService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class ProjectService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListProjects(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/ListProjects',
            depot_dot_core_dot_v1_dot_project__pb2.ListProjectsRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.ListProjectsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/GetProject',
            depot_dot_core_dot_v1_dot_project__pb2.GetProjectRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.GetProjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/CreateProject',
            depot_dot_core_dot_v1_dot_project__pb2.CreateProjectRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.CreateProjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UpdateProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/UpdateProject',
            depot_dot_core_dot_v1_dot_project__pb2.UpdateProjectRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.UpdateProjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/DeleteProject',
            depot_dot_core_dot_v1_dot_project__pb2.DeleteProjectRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.DeleteProjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ResetProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/ResetProject',
            depot_dot_core_dot_v1_dot_project__pb2.ResetProjectRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.ResetProjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListTrustPolicies(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/ListTrustPolicies',
            depot_dot_core_dot_v1_dot_project__pb2.ListTrustPoliciesRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.ListTrustPoliciesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AddTrustPolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/AddTrustPolicy',
            depot_dot_core_dot_v1_dot_project__pb2.AddTrustPolicyRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.AddTrustPolicyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RemoveTrustPolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/RemoveTrustPolicy',
            depot_dot_core_dot_v1_dot_project__pb2.RemoveTrustPolicyRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.RemoveTrustPolicyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListTokens(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/ListTokens',
            depot_dot_core_dot_v1_dot_project__pb2.ListTokensRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.ListTokensResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/CreateToken',
            depot_dot_core_dot_v1_dot_project__pb2.CreateTokenRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.CreateTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UpdateToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/UpdateToken',
            depot_dot_core_dot_v1_dot_project__pb2.UpdateTokenRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.UpdateTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/depot.core.v1.ProjectService/DeleteToken',
            depot_dot_core_dot_v1_dot_project__pb2.DeleteTokenRequest.SerializeToString,
            depot_dot_core_dot_v1_dot_project__pb2.DeleteTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import pytest
import grpc
from unittest.mock import Mock, AsyncMock, patch
from datetime import datetime
from google.protobuf.timestamp_pb2 import Timestamp

from depot_client.api.depot.build.v1 import build_pb2
from depot_client.api.depot.buildkit.v1 import buildkit_pb2
from depot_client.api.depot.core.v1 import build_pb2 as core_build_pb2
from depot_client.client import Client, AsyncClient, BuildEndpoint


def create_timestamp(dt: datetime) -> Timestamp:
    ts = Timestamp()
    ts.FromDatetime(dt)
    return ts


class TestSyncClient:
    @pytest.fixture
    def client(self):
        with patch("grpc.secure_channel") as mock_channel:
            client = Client("test.depot.dev")
            mock_channel.assert_called_once_with(
                "test.depot.dev:443", grpc.ssl_channel_credentials()
            )
            yield client

    def test_create_build(self, client):
        expected_build_id = "build-123"
        expected_token = "token-456"

        client.build.CreateBuild.return_value = build_pb2.CreateBuildResponse(
            build_id=expected_build_id, build_token=expected_token
        )

        build_id, token = client.create_build("project-123")

        assert build_id == expected_build_id
        assert token == expected_token
        client.build.CreateBuild.assert_called_once_with(
            build_pb2.CreateBuildRequest(project_id="project-123")
        )

    def test_finish_build_success(self, client):
        client.finish_build("build-123")

        client.build.FinishBuild.assert_called_once()
        request = client.build.FinishBuild.call_args[0][0]
        assert request.build_id == "build-123"
        assert request.HasField("success")

    def test_finish_build_error(self, client):
        client.finish_build("build-123", error="Something went wrong")

        client.build.FinishBuild.assert_called_once()
        request = client.build.FinishBuild.call_args[0][0]
        assert request.build_id == "build-123"
        assert request.HasField("error")
        assert request.error.error == "Something went wrong"

    def test_get_endpoint(self, client):
        response = buildkit_pb2.GetEndpointResponse(
            active=buildkit_pb2.GetEndpointResponse.ActiveConnection(
                endpoint="tcp://1.2.3.4:1234",
                server_name="test-server",
                cert=buildkit_pb2.CertificatePair(
                    cert=buildkit_pb2.PublicCertificate(cert="client-cert"),
                    key=buildkit_pb2.PrivateKey(key="client-key"),
                ),
                ca_cert=buildkit_pb2.PublicCertificate(cert="ca-cert"),
            )
        )
        client.buildkit.GetEndpoint.return_value = [response]

        endpoints = list(client.get_endpoint("build-123"))
        assert len(endpoints) == 1
        endpoint = endpoints[0]
        assert isinstance(endpoint, BuildEndpoint)
        assert endpoint.endpoint == "tcp://1.2.3.4:1234"
        assert endpoint.server_name == "test-server"
        assert endpoint.client_cert == "client-cert"
        assert endpoint.client_key == "client-key"
        assert endpoint.ca_cert == "ca-cert"

    def test_get_build(self, client):
        now = datetime.now()
        response = core_build_pb2.GetBuildResponse(
            build=core_build_pb2.Build(
                build_id="build-123",
                status=core_build_pb2.Build.STATUS_SUCCESS,
                created_at=create_timestamp(now),
                build_duration_seconds=60,
            )
        )
        client.core_build.GetBuild.return_value = response

        build = client.get_build("build-123")

        assert build["build_id"] == "build-123"
        assert build["status"] == "STATUS_SUCCESS"
        assert build["created_at"].timestamp() == pytest.approx(now.timestamp())
        assert build["build_duration_seconds"] == 60


class TestAsyncClient:
    @pytest.fixture
    async def client(self):
        with patch("grpc.aio.secure_channel") as mock_channel:
            client = AsyncClient("test.depot.dev")
            mock_channel.assert_called_once()
            args = mock_channel.call_args[0]
            assert args[0] == "test.depot.dev:443"
            assert isinstance(args[1], grpc.ChannelCredentials)
            yield client
            await client.close()

    @pytest.mark.asyncio
    async def test_create_build(self, client):
        expected_build_id = "build-123"
        expected_token = "token-456"

        client.build.CreateBuild = AsyncMock(
            return_value=build_pb2.CreateBuildResponse(
                build_id=expected_build_id, build_token=expected_token
            )
        )

        build_id, token = await client.create_build("project-123")

        assert build_id == expected_build_id
        assert token == expected_token
        client.build.CreateBuild.assert_called_once_with(
            build_pb2.CreateBuildRequest(project_id="project-123")
        )

    @pytest.mark.asyncio
    async def test_finish_build_success(self, client):
        client.build.FinishBuild = AsyncMock()
        await client.finish_build("build-123")

        client.build.FinishBuild.assert_called_once()
        request = client.build.FinishBuild.call_args[0][0]
        assert request.build_id == "build-123"
        assert request.HasField("success")

    @pytest.mark.asyncio
    async def test_finish_build_error(self, client):
        client.build.FinishBuild = AsyncMock()
        await client.finish_build("build-123", error="Something went wrong")

        client.build.FinishBuild.assert_called_once()
        request = client.build.FinishBuild.call_args[0][0]
        assert request.build_id == "build-123"
        assert request.HasField("error")
        assert request.error.error == "Something went wrong"

    @pytest.mark.asyncio
    async def test_get_endpoint(self, client):
        response = buildkit_pb2.GetEndpointResponse(
            active=buildkit_pb2.GetEndpointResponse.ActiveConnection(
                endpoint="tcp://1.2.3.4:1234",
                server_name="test-server",
                cert=buildkit_pb2.CertificatePair(
                    cert=buildkit_pb2.PublicCertificate(cert="client-cert"),
                    key=buildkit_pb2.PrivateKey(key="client-key"),
                ),
                ca_cert=buildkit_pb2.PublicCertificate(cert="ca-cert"),
            )
        )

        # Mock async iterator
        async def mock_get_endpoint(_):
            yield response

        client.buildkit.GetEndpoint = Mock(return_value=mock_get_endpoint(None))

        endpoints = []
        async for endpoint in client.get_endpoint("build-123"):
            endpoints.append(endpoint)

        assert len(endpoints) == 1
        endpoint = endpoints[0]
        assert isinstance(endpoint, BuildEndpoint)
        assert endpoint.endpoint == "tcp://1.2.3.4:1234"
        assert endpoint.server_name == "test-server"
        assert endpoint.client_cert == "client-cert"
        assert endpoint.client_key == "client-key"
        assert endpoint.ca_cert == "ca-cert"

    @pytest.mark.asyncio
    async def test_get_build(self, client):
        now = datetime.now()
        response = core_build_pb2.GetBuildResponse(
            build=core_build_pb2.Build(
                build_id="build-123",
                status=core_build_pb2.Build.STATUS_SUCCESS,
                created_at=create_timestamp(now),
                build_duration_seconds=60,
            )
        )
        client.core_build.GetBuild = AsyncMock(return_value=response)

        build = await client.get_build("build-123")

        assert build["build_id"] == "build-123"
        assert build["status"] == "STATUS_SUCCESS"
        assert build["created_at"].timestamp() == pytest.approx(now.timestamp())
        assert build["build_duration_seconds"] == 60
